"""Shardate: Fast reader for YMD-partitioned Parquet files."""

import importlib.metadata

from .shardate import Shardate

try:
    __version__ = importlib.metadata.version("shardate")
except importlib.metadata.PackageNotFoundError:
    __version__ = "unknown"

__all__ = ["Shardate"]




from datetime import date
from typing import Iterable

import pytest
from pyspark.sql import SparkSession
from pyspark.sql import DataFrame

from shardate.shardate import Shardate
from shardate.dates import all_dates_between
from shardate.utils import date_col


@pytest.fixture(scope="session")
def spark():
    """Create a SparkSession for testing."""
    return SparkSession.builder.getOrCreate()


@pytest.fixture(scope="session")
def start_date() -> date:
    return date(2025, 12, 31)


@pytest.fixture(scope="session")
def end_date() -> date:
    return date(2026, 1, 3)


@pytest.fixture(scope="session")
def target_date() -> date:
    return date(2026, 1, 1)


@pytest.fixture(scope="session")
def target_dates() -> list[date]:
    return [date(2025, 12, 31), date(2026, 1, 1), date(2026, 1, 2)]


@pytest.fixture(scope="session")
def path(spark, start_date, end_date, tmp_path_factory):
    """Create sample YMD-partitioned parquet data for testing."""
    df = spark.createDataFrame(
        [
            {
                "id": index,
                "value": f"data_{index}",
                "y": dt.strftime("%Y"),
                "m": dt.strftime("%m"),
                "d": dt.strftime("%d"),
            }
            for index, dt in enumerate(all_dates_between(start_date, end_date))
        ]
    )
    path = tmp_path_factory.mktemp("test_data")
    df.write.parquet(str(path), partitionBy=["y", "m", "d"], mode="overwrite")
    return path


def _get_dates_in_df(df: DataFrame) -> Iterable[date]:
    return [row["date"] for row in sorted(df.select(date_col()).distinct().collect())]


def test_read_by_date(path: str, target_date: date):
    df = Shardate(path).read_by_date(target_date)
    dates = _get_dates_in_df(df)
    assert len(dates) == 1
    assert dates[0] == target_date


def test_read_between(path: str, start_date: date, end_date: date):
    df = Shardate(path).read_between(start_date, end_date)
    dates = _get_dates_in_df(df)
    assert dates == list(all_dates_between(start_date, end_date))


def test_read_by_dates(path: str, target_dates: Iterable[date]):
    df = Shardate(path).read_by_dates(target_dates)
    dates = _get_dates_in_df(df)
    assert dates == sorted(target_dates)


def test_read_eoms_between(path: str, start_date: date, end_date: date):
    df = Shardate(path).read_eoms_between(start_date, end_date)
    dates = _get_dates_in_df(df)
    expected_dates = [
        dt for dt in all_dates_between(start_date, end_date) if dt.day == 31
    ]
    assert dates == expected_dates


def test_read_by_dates(path: str, target_dates: Iterable[date]):
    df = Shardate(path).read_by_dates(target_dates)
    dates = _get_dates_in_df(df)
    assert dates == target_dates



from dateutil.relativedelta import relativedelta


def all_dates_between(start_date, end_date):
    return (
        start_date + relativedelta(days=i)
        for i in range((end_date - start_date).days + 1)
    )


def is_end_of_month(dt):
    return dt == dt + relativedelta(day=1, months=1, days=-1)


def eoms_between(start_date, end_date):
    return [i for i in all_dates_between(start_date, end_date) if is_end_of_month(i)]



from pyspark.sql import SparkSession
from dataclasses import dataclass
from shardate.dates import all_dates_between, eoms_between


def spark_session():
    try:
        return SparkSession.getActiveSession()
    except Exception as e:
        raise RuntimeError("No active SparkSession found. Please create one.") from e


@dataclass
class Shardate:
    path: str
    partition_format: str = "y=%Y/m=%m/d=%d"

    @property
    def read_parquet(self):
        return spark_session().read.options(basePath=self.path).parquet

    def read_by_date(self, target_date):
        return self.read_parquet(
            f"{self.path}/{target_date.strftime(self.partition_format)}"
        )

    def read_between(self, start_date, end_date):
        return self.read_parquet(
            *{
                f"{self.path}/{dt.strftime(self.partition_format)}"
                for dt in all_dates_between(start_date, end_date)
            }
        )

    def read_by_dates(self, target_dates):
        return self.read_parquet(
            *{
                f"{self.path}/{dt.strftime(self.partition_format)}"
                for dt in target_dates
            }
        )

    def read_eoms_between(self, start_date, end_date):
        return self.read_by_dates(eoms_between(start_date, end_date))
